using Newtonsoft.Json;
using System;
using System.Configuration;
using System.IO;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using System.Linq;
using REST_Add_Record_Sample;
using System.Net;

namespace REST_Add_Record_Sample
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                //example of sending a single operation call

                //instantiating LeadRegistrationOperationCall class and assigning properties
                LeadRegistrationOperationCall new_lead_registration = new LeadRegistrationOperationCall()
                {
                    //unique ExternalTransactionID allows to avoid processing of duplicated execute requests
                    //in case ExternalTransactionId is not a part of outgoing JSON - request will be processed without deduplication check
                    //ExternalTransactionId = "",

                    //It is possible to set custom recordID.
                    //Duplication of recordIds will lead to submitted data loss
                    //RecordID = ""

                    //app and operation properties are to be set for multiple operations calls only
                    //app = ""
                    //operation = ""

                    Company = "ABC Inc",
                    FirstName = "John",
                    LastName = "Smith",
                    EmailAddress = "jsmith@example.com",
                    BusinessPhone = "555-7777",
                    MobilePhone = "555-8888",
                    CallbackRequested = new CheckboxesField() { Selection = new string[] { "Yes" } },
                    FirstContact = DateTime.Now,
                    Industry = new SelectionField() { Selection = "Energy" },
                    Rating = new SelectionField() { Selection = "Warm" },
                    TotalValue = 99,
                    EmployeeCount = 25,
                    Probability = new SelectionField() { Selection = "50-80%" },
                    Summary = "Interested in the product. Get in touch in 2 weeks."
                };

                //setting URL of CaptureLead operation REST handler
                string single_operation_rest_url = Path.Combine(ConfigurationManager.AppSettings["base_url"], "LeadRegistrationSample\\CaptureLead");

                //serializing new_lead_registration object to JSON
                string content = JsonConvert.SerializeObject(new_lead_registration, Formatting.Indented);

                ServicePointManager.ServerCertificateValidationCallback += (sender, cert, chain, sslPolicyErrors) => true;

                //posting async request
                HttpResponseMessage response = DispatchPostRequest(single_operation_rest_url, content).GetAwaiter().GetResult();

                //checking execute response status code 
                if (response.StatusCode == System.Net.HttpStatusCode.OK)
                {
                    //execute is successfull. Deserializing response to  LeadRegistrationSingleOperationResponse object and printing status code
                    LeadRegistrationSingleOperationResponse executeResponse = JsonConvert.DeserializeObject<LeadRegistrationSingleOperationResponse>(response.Content.ReadAsStringAsync().Result);
                    Console.WriteLine(executeResponse.status);
                }
                else
                {
                    //execute is faulty - extracting error message from "x-error-msg" header                    
                    Console.WriteLine(response.Headers.GetValues("x-error-msg").First());
                }

                Console.WriteLine("*********************************");
                //**********************************************
                //Example of sending multiple operation calls

                //instantiating LeadRegistration class and assigning properties
                LeadRegistrationOperationCall new_lead_registration_call_1 = new LeadRegistrationOperationCall()
                {
                    Company = "ABC Inc",
                    FirstName = "John",
                    LastName = "Smith",
                    EmailAddress = "jsmith@example.com",
                    BusinessPhone = "555-7777",
                    MobilePhone = "555-8888",
                    CallbackRequested = new CheckboxesField() { Selection = new string[] { "Yes" } },
                    FirstContact = DateTime.Now,
                    Industry = new SelectionField() { Selection = "Energy" },
                    Rating = new SelectionField() { Selection = "Warm" },
                    TotalValue = 99,
                    EmployeeCount = 25,
                    Probability = new SelectionField() { Selection = "50-80%" },
                    Summary = "Interested in the product. Get in touch in 2 weeks."
                };

                //instantiating LeadRegistration class and assigning properties
                LeadRegistrationOperationCall new_lead_registration_call_2 = new LeadRegistrationOperationCall()
                {
                    Company = "ABC Inc",
                    FirstName = "Jane",
                    LastName = "Smith",
                    EmailAddress = "jsmith@example.com",
                    BusinessPhone = "555-7788",
                    MobilePhone = "555-8899",
                    CallbackRequested = new CheckboxesField() { Selection = new string[] { "Yes" } },
                    FirstContact = DateTime.Now,
                    Industry = new SelectionField() { Selection = "Energy" },
                    Rating = new SelectionField() { Selection = "Warm" },
                    TotalValue = 99,
                    EmployeeCount = 25,
                    Probability = new SelectionField() { Selection = "50-80%" },
                    Summary = "Interested in the product. Get in touch in 2 weeks."
                };

                OperationCalls opCalls = new OperationCalls()
                {
                    operationCalls = new OperationCall[] { new_lead_registration_call_1, new_lead_registration_call_2 }
                };

                //setting URL of CaptureLead operation REST handler
                string multiple_operations_rest_url = ConfigurationManager.AppSettings["base_url"];

                //serializing new_lead_registration object to JSON
                content = JsonConvert.SerializeObject(opCalls, Formatting.Indented);                

                //posting async request
                response = DispatchPostRequest(multiple_operations_rest_url, content).GetAwaiter().GetResult();

                //checking execute response status code 
                if (response.StatusCode == System.Net.HttpStatusCode.OK)
                {
                    //execute is successfull. Deserializing response to  LeadRegistrationSingleOperationResponse object and printing status code
                    var qwe = response.Content.ReadAsStringAsync().Result;
                    LeadRegistrationMultipleOperationsResponse executeResponse = JsonConvert.DeserializeObject<LeadRegistrationMultipleOperationsResponse>(response.Content.ReadAsStringAsync().Result);
                    Console.WriteLine(executeResponse.status);
                }
                else
                {
                    //execute is faulty - extracting error message from "x-error-msg" header                    
                    Console.WriteLine(response.Headers.GetValues("x-error-msg").First());
                }

            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
            finally
            {
                Console.WriteLine(Environment.NewLine + "Press any key to exit");
                Console.ReadKey();
            }
        }

        public static string JsonPrettify(string json)
        {
            using (var stringReader = new StringReader(json))
            using (var stringWriter = new StringWriter())
            {
                var jsonReader = new JsonTextReader(stringReader);
                var jsonWriter = new JsonTextWriter(stringWriter) { Formatting = Formatting.Indented };
                jsonWriter.WriteToken(jsonReader);
                return stringWriter.ToString();
            }
        }

        public static async Task<HttpResponseMessage> DispatchPostRequest(string capture_lead_operation_rest_url, string content)
        {
            //instantiating HttpClient within the try with resources construction
            using (HttpClient client = new HttpClient())
            {
                //setting Basic authorization credentials
                client.DefaultRequestHeaders.Authorization =
                    new AuthenticationHeaderValue(
                        "Basic", Convert.ToBase64String(
                            System.Text.ASCIIEncoding.ASCII.GetBytes(ConfigurationManager.AppSettings["rest_login"] + ":" + ConfigurationManager.AppSettings["rest_password"])));

                //posting serialized new_lead_registration object to capture_lead_operation_rest_url 
                return await client.PostAsync(capture_lead_operation_rest_url, new StringContent(content));
            }
        }
    }

    //description of LeadRegistration record class
    public class LeadRegistrationRecord
    {
        public string Company { get; set; }
        public string FirstName { get; set; }
        public string LastName { get; set; }
        public string EmailAddress { get; set; }
        public string BusinessPhone { get; set; }
        public string MobilePhone { get; set; }
        public CheckboxesField CallbackRequested { get; set; }
        public DateTime FirstContact { get; set; }
        public SelectionField Industry { get; set; }
        public SelectionField Rating { get; set; }
        public int TotalValue { get; set; }
        public int EmployeeCount { get; set; }
        public SelectionField Probability { get; set; }
        public string Summary { get; set; }
    }

    //description of Checkboxes field
    public class CheckboxesField
    {
        public string[] Selection;
    }

    public class LocationField
    {
        public decimal Latitude;
        public decimal Longitude;
    }

    //description of selection fields (radio button/dropdown fields) class
    public class SelectionField
    {
        public string Selection;
    }

    //description of ExecuteResponse class for LeadRegistration app 
    public class LeadRegistrationSingleOperationResponse : LeadRegistrationRecord, ExecuteResponse
    {
        public int status { get; set; }
        public string ExternalTransactionId { get; set; }
        public string RecordID { get; set; }
        public DateTime ClientTime { get; set; }
        public string CreatedBy { get; set; }
        public DateTime CreatedOn { get; set; }
        public string ModifiedBy { get; set; }
        public DateTime ModifiedOn { get; set; }
        public string LastOperationName { get; set; }
        public string LastSubmissionID { get; set; }
        public string State { get; set; }
        public LocationField ClientLocation { get; set; }
    }

    public class LeadRegistrationMultipleOperationsResponse
    {
        public int status { get; set; }
        public LeadRegistrationSingleOperationResponse[] Operations { get; set; }
    }

    public interface ExecuteResponse
    {
        int status { get; set; }
        string ExternalTransactionId { get; set; }
        string RecordID { get; set; }
        DateTime ClientTime { get; set; }
        string CreatedBy { get; set; }
        DateTime CreatedOn { get; set; }
        string ModifiedBy { get; set; }
        DateTime ModifiedOn { get; set; }
        string LastOperationName { get; set; }
        string LastSubmissionID { get; set; }
        string State { get; set; }
        LocationField ClientLocation { get; set; }
    }

    public class OperationCalls
    {
        public OperationCall[] operationCalls { get; set; }
    }

    public interface OperationCall
    {
        string app { get; set; }
        string operation { get; set; }
        string RecordID { get; set; }
        string ExternalTransactionId { get; set; }
    }

    public class LeadRegistrationOperationCall : LeadRegistrationRecord, OperationCall
    {
        public string app { get; set; }
        public string operation { get; set; }
        public string RecordID { get; set; }
        public string ExternalTransactionId { get; set; }
    }
}
