using Newtonsoft.Json;
using System;
using System.Configuration;
using System.IO;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using System.Web;

namespace REST_View_Query_Sample
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                //Example of retrieving data from View All Leads REST enabled view

                //setting URL of View All Leads REST enabled view
                string view_all_leads_rest_url = Path.Combine(ConfigurationManager.AppSettings["base_url"], "LeadRegistrationSample/ViewAllLeads");

                //sending GET request to REST handler and obtaining JSON with array of records
                string view_all_leads_content_json = DispatchGetRequest(view_all_leads_rest_url).GetAwaiter().GetResult().Content.ReadAsStringAsync().Result;

                //Writing prettified result to Console
                Console.WriteLine(JsonPrettify(view_all_leads_content_json));

                //Deserializing JSON to array of LeadRegistration objects              
                LeadRegistration[] all_leads_record_objects = JsonConvert.DeserializeObject<LeadRegistration[]>(view_all_leads_content_json);

                Console.WriteLine("*********************************");
                //**********************************************
                //Example of retrieving records from SynchronizeRecords REST enabled parametrized view
                

                //setting URL of SynchronizeRecords REST enabled parametrized view
                string synchronizeRecords_rest_url = Path.Combine(ConfigurationManager.AppSettings["base_url"], "LeadRegistrationSample/SynchronizeRecords");

                //getting values of PreviousLastSubmissionID REST parameter 
                var previousLastSubmissionIDPath = ConfigurationManager.AppSettings["PreviousLastSubmissionIDPath"];
                string PreviousLastSubmissionID = GetPreviousLastSubmissionID(previousLastSubmissionIDPath);

                //creation of combined URL with parameters                 
                string combined_url_with_parameters = String.Format(synchronizeRecords_rest_url + "?PreviousLastSubmissionID={0}", PreviousLastSubmissionID);

                //sending GET request to REST handler and obtaining JSON with array of records
                string synchronizeRecords_content_json = DispatchGetRequest(combined_url_with_parameters).GetAwaiter().GetResult().Content.ReadAsStringAsync().Result;                

                //Writing prettified result to Console
                Console.WriteLine(JsonPrettify(synchronizeRecords_content_json));                

                //Deserializing JSON to array of LeadRegistration objects              
                LeadRegistration[] records = JsonConvert.DeserializeObject<LeadRegistration[]>(synchronizeRecords_content_json);
                Console.WriteLine(Environment.NewLine + String.Format("Received {0} records", records != null ? records.Length : 0));

                //Extracting LastSubmissionID from the last record
                string currentLastSubmissionID = null;
                if (records != null && records.Length > 0)
                {
                    currentLastSubmissionID = records[records.Length - 1].LastSubmissionID;
                }

                if (!string.IsNullOrEmpty(currentLastSubmissionID)) SetLastSubmissionID(previousLastSubmissionIDPath, currentLastSubmissionID);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
            finally
            {
                Console.WriteLine(Environment.NewLine + "Press any key to exit");
                Console.ReadKey();
            }
        }

        public static void SetLastSubmissionID(string path, string LastSubmissionID)
        {
            Console.WriteLine(String.Format("Saving LastSubmissionID '{0}' to '{1}'", LastSubmissionID, path));
            File.WriteAllText(path, LastSubmissionID);
        }

        public static string GetPreviousLastSubmissionID(string path)
        {
            if(File.Exists(path))
            {
                return File.ReadAllText(path);
            }
            return null;
        }

        public static async Task<HttpResponseMessage> DispatchGetRequest(string view_rest_url)
        {
            //instantiating HttpClient within the try with resources construction
            using (HttpClient client = new HttpClient())
            {
                //setting Basic authorization credentials
                client.DefaultRequestHeaders.Authorization =
                    new AuthenticationHeaderValue(
                        "Basic", Convert.ToBase64String(
                            System.Text.ASCIIEncoding.ASCII.GetBytes(ConfigurationManager.AppSettings["rest_login"] + ":" + ConfigurationManager.AppSettings["rest_password"])));
                
                return await client.GetAsync(view_rest_url);
            }
        }

        public static string JsonPrettify(string json)
        {
            using (var stringReader = new StringReader(json))
            using (var stringWriter = new StringWriter())
            {
                var jsonReader = new JsonTextReader(stringReader);
                var jsonWriter = new JsonTextWriter(stringWriter) { Formatting = Formatting.Indented };
                jsonWriter.WriteToken(jsonReader);
                return stringWriter.ToString();
            }
        }
    }

    //description of LeadRegistration record class
    public class LeadRegistration
    {
        public string Company { get; set; }
        public string FirstName { get; set; }
        public string LastName { get; set; }
        public string EmailAddress { get; set; }
        public string BusinessPhone { get; set; }
        public string MobilePhone { get; set; }
        public CheckboxesField CallbackRequested { get; set; }
        public DateTime? FirstContact { get; set; }
        public SelectionField Industry { get; set; }
        public SelectionField Rating { get; set; }
        public int TotalValue { get; set; }
        public int EmployeeCount { get; set; }
        public SelectionField Probability { get; set; }
        public string Summary { get; set; }
        public DateTime? ClientTime { get; set; }
        public string CreatedBy { get; set; }
        public DateTime? CreatedOn { get; set; }
        public string LastOperationName { get; set; }
        public string ModifiedBy { get; set; }
        public DateTime? ModifiedOn { get; set; }
        public string RecordID { get; set; }
        public string State { get; set; }
        public string LastSubmissionID { get; set; }
    }

    //description of Checkboxes field class
    public class CheckboxesField
    {
        public string[] Selection;
    }

    //description of selection fields (radio button/dropdown fields) class
    public class SelectionField
    {
        public string Selection;
    }

    //description of location field class
    public class LocationField
    {
        public string Latitude { get; set; }
        public string Longitude { get; set; }
    }
}
